'/*
' * @(#)AUDIO.java - parse Audioheaders
' *
' * Copyright (c) 2003-2004 by dvb.matt, All Rights Reserved.
' * 
' * This file is part of X, a free Java based demux utility.
' * X is intended for educational purposes only, as a non-commercial test project.
' * It may not be used otherwise. Most parts are only experimental.
' * 
' * The part of audio parsing was derived from the MPEG/Audio
' * Software Simulation Group's audio codec and ATSC A/52 in a special modified manner.
' *
' *
' * This program is free software; you can redistribute it free of charge
' * and/or modify it under the terms of the GNU General friend License as published by
' * the Free Software Foundation; either version 2 of the License, or
' * (at your option) any later version.
' *
' * This program is distributed in the hope that it will be useful,
' * but WITHOUT ANY WARRANTY; without even the implied warranty of
' * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' * GNU General friend License for more details.
' *
' * You should have received a copy of the GNU General friend License
' * along with this program; if not, write to the Free Software
' * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
' *
' */
Public Enum AudioFormatType
    AC3
    MPA
    DTS
    WAV
End Enum

Public Structure AudioInfo
    Public Format As AudioFormatType
    Public FormatText As String
    Public Channels As String
    Public Bitrate As Integer 'in kbps
    Public Frequence As Integer

    Public Overrides Function ToString() As String
        Return String.Format("{0}, {1}, {2}kbps, {3:#,##0}Hz", Me.FormatText, Me.Channels, Me.Bitrate, Me.Frequence)
    End Function
End Structure

Friend Class AudioBuffer
    Friend ID As Integer = 0
    Friend Layer As Integer = 0
    Friend Protection_bit As Integer = 0
    Friend Bitrate As Integer = 0
    Friend Sampling_frequency As Integer = 0
    Friend Padding_bit As Integer = 0
    Friend Private_bit As Integer = 0
    Private Mode As Integer = 0
    Friend Mode_extension As Integer = 0
    Friend Copyright As Integer = 0
    Friend Original As Integer = 0
    Friend Channel As Integer = 0
    Friend Emphasis As Integer = 0
    Friend Size As Integer = 0
    Friend Size_base As Integer = 0
    Friend Bound As Integer = 0 '//DM10042004 081.7 int01 add
    Friend Sblimit As Integer '//DM10042004 081.7 int01 add
    Friend Time_length As Double = 0.0

    Friend nID As Integer = 0
    Friend nLayer As Integer = 0
    Friend nProtection_bit As Integer = 0
    Friend nBitrate As Integer = 0
    Friend nSampling_frequency As Integer = 0
    Friend nPadding_bit As Integer = 0
    Friend nprivate_bit As Integer = 0
    Friend nMode As Integer = 0
    Friend nMode_extension As Integer = 0
    Friend nCopyright As Integer = 0
    Friend nOriginal As Integer = 0
    Friend nChannel As Integer = 0
    Friend nEmphasis As Integer = 0
    Friend nSize As Integer = 0
    Friend nSize_base As Integer = 0
    Friend nTime_length As Double = 0

    Private ReadOnly mpa_bitrate_index(,,) As Integer = {{ _
     {-1, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, _
     80000, 96000, 112000, 128000, 144000, 160000, 0}, _
     {-1, 8000, 16000, 24000, 32000, 40000, 48000, 56000, 64000, _
     80000, 96000, 112000, 128000, 144000, 160000, 0}, _
     {-1, 32000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, _
     144000, 160000, 176000, 192000, 224000, 256000, 0} _
    }, { _
     {-1, 32000, 40000, 48000, 56000, 64000, 80000, 96000, _
     112000, 128000, 160000, 192000, 224000, 256000, 320000, 0}, _
     {-1, 32000, 48000, 56000, 64000, 80000, 96000, 112000, _
     128000, 160000, 192000, 224000, 256000, 320000, 384000, 0}, _
     {-1, 32000, 64000, 96000, 128000, 160000, 192000, 224000, _
     256000, 288000, 320000, 352000, 384000, 416000, 448000, 0} _
    }, { _
     {-1, 6000, 8000, 10000, 12000, 16000, 20000, 24000, _
     28000, 320000, 40000, 48000, 56000, 64000, 80000, 0}, _
     {-1, 6000, 8000, 10000, 12000, 16000, 20000, 24000, _
     28000, 320000, 40000, 48000, 56000, 64000, 80000, 0}, _
     {-1, 8000, 12000, 16000, 20000, 24000, 32000, 40000, _
     48000, 560000, 64000, 80000, 96000, 112000, 128000, 0} _
    }}

    Private ReadOnly mpa_frequency_index(,) As Integer = { _
     {22050, 24000, 16000, 0}, _
     {44100, 48000, 32000, 0}, _
     {11025, 12000, 8000, 0} _
    }

    Private ReadOnly time_index() As Double = {0.0, 103680000.0, 103680000.0, 34560000.0} '//L3,L2,L1 * 90

    Private ReadOnly mpa_ID() As String = {"MPEG-2", "MPEG-1", "MPEG-2.5"}
    Private ReadOnly mpa_Layer() As String = {"n.a.", "Layer III", "Layer II", "Layer I"}
    Private ReadOnly mpa_CRC() As String = {"noCRC", "CRC"}
    Private ReadOnly mpa_Mode() As String = {"Stereo", "Joint Stereo", "Dual Channel", "Mono"}


    '/*** parse mpa Header ***/
    '//DM10042004 081.7 int01 changed
    Friend Function MPA_parseHeader(ByVal frame() As Byte, ByVal pos As Integer) As Integer
        Dim sblimit As Integer = 32

        If ((&HFF And frame(pos)) <> &HFF OrElse (&HF0 And frame(pos + 1)) <> &HF0) Then Return -1

        ID = 1 And Bitweise.RechtsOhne(frame(pos + 1), 3)
        Emphasis = 3 And frame(pos + 3)

        If (ID = 1 AndAlso Emphasis = 2) Then ID = 2
        Layer = 3 And Bitweise.RechtsOhne(frame(pos + 1), 1)
        If (Layer < 1) Then Return -2

        Protection_bit = (1 And frame(pos + 1)) ^ 1

        Bitrate = mpa_bitrate_index(ID, Layer - 1, &HF And Bitweise.RechtsOhne(frame(pos + 2), 4))
        If (Bitrate < 1) Then Return -3

        Sampling_frequency = mpa_frequency_index(ID, 3 And Bitweise.RechtsOhne(frame(pos + 2), 2))
        If (Sampling_frequency = 0) Then Return -4

        Padding_bit = 1 And Bitweise.RechtsOhne(frame(pos + 2), 1)
        Private_bit = 1 And frame(pos + 2)

        Mode = 3 And Bitweise.RechtsOhne(frame(pos + 3), 6)
        Mode_extension = 3 And Bitweise.RechtsOhne(frame(pos + 3), 4)
        If (Mode = 0) Then Mode_extension = 0

        Bound = IIf(Mode = 1, ((Mode_extension + 1) << 2), sblimit)
        Channel = IIf(Mode = 3, 1, 2)
        Copyright = 1 And Bitweise.RechtsOhne(frame(pos + 3), 3)
        Original = 1 And Bitweise.RechtsOhne(frame(pos + 3), 2)
        Time_length = time_index(Layer) / Sampling_frequency

        If (ID = 1 AndAlso Layer = 2) Then ' {	// MPEG-1, L2 restrictions
            If (Bitrate / Channel < 32000) Then Return -5 '/* unsupported bitrate */
            If (Bitrate / Channel > 192000) Then Return -6 '/* unsupported bitrate */

            If (Bitrate < 56000) Then
                If (Sampling_frequency = 32000) Then
                    sblimit = 12
                Else
                    sblimit = 8
                End If
            ElseIf (Bitrate < 96000) Then
                sblimit = 27
            Else
                If (Sampling_frequency = 48000) Then
                    sblimit = 27
                Else
                    sblimit = 30
                End If
            End If

            If (Bound > sblimit) Then Bound = sblimit
        ElseIf (Layer = 2) Then '// MPEG-2
            sblimit = 30
        End If

        If (Layer < 3) Then
            If (Bound > sblimit) Then Bound = sblimit
            Size_base = Int(144 * Bitrate / Sampling_frequency)
            Size = Size_base + Padding_bit
            Return 1
        Else
            sblimit = 32
            Size_base = Int(12 * Bitrate / Sampling_frequency) * 4
            Size = Size_base + (4 * Padding_bit)
            Return 2
        End If
    End Function

    '/*** parse next mpa Header ***/
    Friend Function MPA_parseNextHeader(ByVal frame() As Byte, ByVal pos As Integer) As Integer

        If ((&HFF And frame(pos)) <> &HFF OrElse (&HF0 And frame(pos + 1)) <> &HF0) Then Return -1

        nID = 1 And Bitweise.RechtsOhne(frame(pos + 1), 3)
        nEmphasis = 3 And frame(pos + 3)

        If (nID = 1 AndAlso nEmphasis = 2) Then nID = 2

        nLayer = 3 And Bitweise.RechtsOhne(frame(pos + 1), 1)
        If (nLayer < 1) Then Return -2

        nProtection_bit = (1 And frame(pos + 1)) ^ 1

        nBitrate = mpa_bitrate_index(nID, nLayer - 1, &HF And Bitweise.RechtsOhne(frame(pos + 2), 4))
        If (nBitrate < 1) Then Return -3

        nSampling_frequency = mpa_frequency_index(nID, 3 And Bitweise.RechtsOhne(frame(pos + 2), 2))
        If (nSampling_frequency = 0) Then Return -4

        nPadding_bit = 1 And Bitweise.RechtsOhne(frame(pos + 2), 1)
        nprivate_bit = 1 And frame(pos + 2)

        nMode = 3 And Bitweise.RechtsOhne(frame(pos + 3), 6)
        nMode_extension = 3 And Bitweise.RechtsOhne(frame(pos + 3), 4)
        If (nMode = 0) Then nMode_extension = 0

        nChannel = IIf(nMode = 3, 1, 2)
        nCopyright = 1 And Bitweise.RechtsOhne(frame(pos + 3), 3)
        nOriginal = 1 And Bitweise.RechtsOhne(frame(pos + 3), 2)
        nTime_length = time_index(nLayer) / nSampling_frequency

        If (nID = 1 AndAlso nLayer = 2) Then '{	// MPEG-1,L2 restrictions
            If (nBitrate / Channel < 32000) Then Return -5 '; /* unsupported bitrate */
            If (nBitrate / Channel > 192000) Then Return -6 '; /* unsupported bitrate */
        End If

        If (nLayer < 3) Then
            nSize_base = Int(144 * nBitrate / nSampling_frequency)
            nSize = nSize_base + nPadding_bit
            Return 1
        Else
            nSize_base = Int(12 * nBitrate / nSampling_frequency) * 4
            nSize = nSize_base + (4 * nPadding_bit)
            Return 2
        End If
    End Function

    'Friend Function MPA_displayHeader() As String
    '    Return ("" & mpa_ID(ID) & ", " & mpa_Layer(Layer) & ", " & Sampling_frequency & "Hz, " & mpa_Mode(Mode) & ", " & (Bitrate / 1000) & "kbps, " & mpa_CRC(Protection_bit))
    'End Function

    ''/*** display last mpa header ***/
    'Friend Function MPA_saveAnddisplayHeader() As String
    '    'saveHeader()
    '    Return MPA_displayHeader()
    'End Function

    Private ReadOnly ac3_frequency_index() As Integer = {48000, 44100, 32000, 0}

    Private ReadOnly ac3_bitrate_index() As Integer = { _
     32000, 40000, 48000, 56000, 64000, 80000, 96000, _
     112000, 128000, 160000, 192000, 224000, 256000, _
     320000, 384000, 448000, 512000, 576000, 640000, _
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 _
    }

    Private ReadOnly ac3_size_table(,) As Integer = { _
     {128, 160, 192, 224, 256, 320, 384, 448, 512, 640, 768, 896, 1024, 1280, 1536, 1792, 2080, 2304, 2560}, _
     {138, 174, 208, 242, 278, 348, 416, 486, 556, 696, 834, 974, 1114, 1392, 1670, 1950, 2228, 2506, 2786}, _
     {192, 240, 288, 336, 384, 480, 576, 672, 768, 960, 1152, 1344, 1536, 1920, 2304, 2688, 3120, 3456, 3840} _
    }

    Private ReadOnly ac3_bsmod() As String = {", CM", ", ME", ", K:VI", ", K:HI", ", K:D", ", K:C", ", K:E", ", K:VO"}
    Private ReadOnly ac3_cmixlev() As String = {",cm-3.0dB", ",cm-4.5dB", ",cm-6.0dB", ",cm-4.5dB"}
    Private ReadOnly ac3_surmixlev() As String = {",sm-3dB", ",sm-6dB", ",sm 0dB", ",sm-6dB"}
    Private ReadOnly ac3_dsurmod() As String = {"", ", notDS", ", DS", ""}

    Private ReadOnly ac3_mod() As String = {"1+1", "1/0", "2/0", "3/0", "2/1", "3/1", "2/2", "3/2"}

    Private ReadOnly ac3_channels() As Integer = {2, 1, 2, 3, 3, 4, 4, 5}

    Private ReadOnly ac3_lfe(,) As String = {{".0", ".1"}, {"", "lfe"}}

    '/*** parse ac3 Header ***/
    Friend Function AC3_parseHeader(ByVal frame() As Byte, ByVal pos As Integer) As Integer
        If ((&HFF And frame(pos)) <> &HB OrElse (&HFF And frame(pos + 1)) <> &H77) Then Return -1

        ID = 0
        Emphasis = 0
        Protection_bit = 0 ^ 1

        Sampling_frequency = ac3_frequency_index(3 And Bitweise.RechtsOhne(frame(pos + 4), 6))
        If (Sampling_frequency < 1) Then Return -4

        Bitrate = ac3_bitrate_index(&H1F And Bitweise.RechtsOhne(frame(pos + 4), 1))
        If (Bitrate < 1) Then Return -3

        Layer = 7 And frame(pos + 5) '       //bsmod
        Padding_bit = 1 And frame(pos + 4)
        Private_bit = 0
        Mode = 7 And Bitweise.RechtsOhne(frame(pos + 6), 5)
        Mode_extension = 0

        Dim intmode As Integer = (&HFF And frame(pos + 6)) << 8 Or (&HFF And frame(pos + 7))
        Dim skip As Integer = 0

        If ((Mode And 1) > 0 AndAlso Mode <> 1) Then skip += 1
        If ((Mode And 4) > 0) Then skip += 1
        If (Mode = 2) Then
            Mode_extension = Mode_extension Or (6 And Bitweise.RechtsOhne(intmode, (10 - (2 * skip)))) 'DS
            skip += 1
        End If

        Select Case (skip) 'lfe
            Case 0
                Mode_extension = Mode_extension Or (1 And Bitweise.RechtsOhne(intmode, 12))
            Case 1
                Mode_extension = Mode_extension Or (1 And Bitweise.RechtsOhne(intmode, 10))
            Case 2
                Mode_extension = Mode_extension Or (1 And Bitweise.RechtsOhne(intmode, 8))
            Case 3
                Mode_extension = Mode_extension Or (1 And Bitweise.RechtsOhne(intmode, 6))
        End Select

        Channel = ac3_channels(Mode) + (1 And Mode_extension)
        Copyright = 0
        Original = 0
        Time_length = 138240000.0 / Sampling_frequency

        Size_base = ac3_size_table(3 And Bitweise.RechtsOhne(frame(pos + 4), 6), &H1F And Bitweise.RechtsOhne(frame(pos + 4), 1))
        Size = Size_base + Padding_bit * 2
        Return 1
    End Function

    '/*** parse ac3 Header ***/
    Friend Function AC3_parseNextHeader(ByVal frame() As Byte, ByVal pos As Integer) As Integer

        If ((&HFF And frame(pos)) <> &HB OrElse (&HFF And frame(pos + 1)) <> &H77) Then Return -1

        nID = 0
        nEmphasis = 0
        nProtection_bit = 0 ^ 1

        nSampling_frequency = ac3_frequency_index(3 And Bitweise.RechtsOhne(frame(pos + 4), 6))
        If (nSampling_frequency < 1) Then Return -4

        nBitrate = ac3_bitrate_index(&H1F And Bitweise.RechtsOhne(frame(pos + 4), 1))
        If (nBitrate < 1) Then Return -3

        nLayer = 7 And frame(pos + 5) '       //bsmod
        nPadding_bit = 1 And frame(pos + 4)
        nprivate_bit = 0

        nMode = 7 And Bitweise.RechtsOhne(frame(pos + 6), 5)
        Dim intmode As Integer = (&HFF And frame(pos + 6)) << 8 Or (&HFF And frame(pos + 7))
        Dim skip As Integer = 0

        If ((nMode And 1) > 0 AndAlso nMode <> 1) Then skip += 1 '//cmix

        If ((nMode And 4) > 0) Then skip += 1 'surmix

        If (nMode = 2) Then ' //DS mode
            nMode_extension = nMode_extension Or 6 And Bitweise.RechtsOhne(intmode, (10 - (2 * skip))) '//DS
            skip += 1
        End If

        Select Case (skip) '//lfe
            Case 0
                nMode_extension = nMode_extension Or 1 And Bitweise.RechtsOhne(intmode, 12)
            Case 1
                nMode_extension = nMode_extension Or 1 And Bitweise.RechtsOhne(intmode, 10)
            Case 2
                nMode_extension = nMode_extension Or 1 And Bitweise.RechtsOhne(intmode, 8)
            Case 3
                nMode_extension = nMode_extension Or 1 And Bitweise.RechtsOhne(intmode, 6)
        End Select

        nChannel = ac3_channels(Mode) + (1 And nMode_extension)
        nCopyright = 0
        nOriginal = 0
        nTime_length = 138240000.0 / nSampling_frequency

        nSize_base = ac3_size_table(3 And Bitweise.RechtsOhne(frame(pos + 4), 6), 5 And Bitweise.RechtsOhne(frame(pos + 4), 1))
        nSize = nSize_base + nPadding_bit * 2
        Return 1
    End Function




    ''/*** display last ac3 header ***/
    'Friend Function AC3_displayHeader() As String
    '    Return ("AC-3" & ac3_bsmod(Layer) & ", " & ac3_mod(Mode) & ac3_lfe(1, 1 And Mode_extension) & "(" & ac3_channels(Mode) & ac3_lfe(0, 1 And Mode_extension) & ")" & ac3_dsurmod(Bitweise.RechtsOhne(Mode_extension, 1)) & ", " & Sampling_frequency & "Hz, " & (Bitrate / 1000) & "kbps")
    'End Function

    ''/*** display last ac3 header ***/
    'Friend Function AC3_saveAnddisplayHeader() As String
    '    'saveHeader()
    '    Return AC3_displayHeader()
    'End Function

    '//start 081.6 int07
    '/* DTS stuff taken from the VideoLAN project. */ 
    '/* Added by R One, 2003/12/18. */ 
    Private ReadOnly dts_frequency_index() As Integer = { _
     0, 8000, 16000, 32000, 64000, 128000, _
     11025, 22050, 44100, 88200, 176400, _
     12000, 24000, 48000, 96000, 192000 _
    }

    Private ReadOnly dts_bitrate_index() As Integer = { _
     32000, 56000, 64000, 96000, 112000, 128000, _
     192000, 224000, 256000, 320000, 384000, _
     448000, 512000, 576000, 640000, 768000, _
     896000, 1024000, 1152000, 1280000, 1344000, _
     1408000, 1411200, 1472000, 1536000, 1920000, _
     2048000, 3072000, 3840000, 4096000, 0, 0 _
    }

    Private ReadOnly dts_acmod() As String = { _
     "1", "DM", "2/0", "2/0", _
     "2/0", "3/0", "2.1/0", "3.1/0", _
     "2/2", "3/2", "2/2/2", "2/2/2", _
     "3/2/2", "3.1/2/2", "", "", _
     "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", _
     "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", _
     "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" _
    }

    Private ReadOnly dts_channels() As Integer = { _
     1, 2, 2, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 0, 0, _
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 _
    }

    '/*** parse dts Header ***/ 
    Friend Function DTS_parseHeader(ByVal frame() As Byte, ByVal pos As Integer) As Integer

        If (frame(pos) <> &H7F OrElse frame(pos + 1) <> &HFE OrElse frame(pos + 2) <> &H80 OrElse frame(pos + 3) <> 1) Then Return -1

        ID = 0
        Emphasis = 0
        Protection_bit = 0 ^ 1

        If ((Sampling_frequency = dts_frequency_index(&HF And (Bitweise.RechtsOhne(frame(pos + 8), 2)))) < 1) Then Return -4
        Bitrate = dts_bitrate_index(((3 And frame(pos + 8)) << 3) Or (7 And (Bitweise.RechtsOhne(frame(pos + 9), 5))))
        If (Bitrate < 1) Then Return -3

        Layer = 0
        Padding_bit = 0
        Private_bit = 0


        Mode = ((&HF And frame(pos + 7)) << 2) Or Bitweise.RechtsOhne((&HC0 And frame(pos + 8)), 6)
        Mode_extension = 0
        Channel = dts_channels(Mode)
        Copyright = 0
        Original = 0
        Size = ((1 And frame(pos + 4)) << 6) Or Bitweise.RechtsOhne((&HFC And frame(pos + 5)), 2)
        Size = (Size + 1) << 5
        Time_length = 90000.0 * Size / Sampling_frequency
        Size = ((3 And frame(pos + 5)) << 12) Or ((&HFF And frame(pos + 6)) << 4) Or Bitweise.RechtsOhne((&HF0 And frame(pos + 7)), 4)
        Size += 1
        Size_base = Size
        Return 1
    End Function

    '/*** parse dts Header ***/ 
    Friend Function DTS_parseNextHeader(ByVal frame() As Byte, ByVal pos As Integer) As Integer

        If (frame(pos) <> &H7F OrElse frame(pos + 1) <> &HFE OrElse frame(pos + 2) <> &H80 OrElse frame(pos + 3) <> 1) Then Return -1

        nID = 0
        nEmphasis = 0
        nProtection_bit = 0 ^ 1

        nSampling_frequency = dts_frequency_index(&HF And (Bitweise.RechtsOhne(frame(pos + 8), 2)))
        If (nSampling_frequency < 1) Then Return -4

        nBitrate = dts_bitrate_index(((3 And frame(pos + 8)) << 3) Or (7 And (Bitweise.RechtsOhne(frame(pos + 9), 5))))
        If (nBitrate < 1) Then Return -3

        nLayer = 0
        nPadding_bit = 0
        nprivate_bit = 0

        nMode = ((&HF And frame(pos + 7)) << 2) Or Bitweise.RechtsOhne((&HC0 And frame(pos + 8)), 6)
        nMode_extension = 0
        nChannel = dts_channels(nMode)
        nCopyright = 0
        nOriginal = 0
        nSize = ((1 And frame(pos + 4)) << 6) Or Bitweise.RechtsOhne((&HFC And frame(pos + 5)), 2)
        nSize = (nSize + 1) << 5
        nTime_length = 90000.0 * nSize / nSampling_frequency
        nSize = ((3 And frame(pos + 5)) << 12) Or ((&HFF And frame(pos + 6)) << 4) Or Bitweise.RechtsOhne((&HF0 And frame(pos + 7)), 4)
        nSize += 1
        nSize_base = nSize
        Return 1
    End Function


    ''/*** display last dts header ***/ 
    'Friend Function DTS_displayHeader() As String
    '    Return ("DTS, " + dts_acmod(Mode) + "(" + dts_channels(Mode) + "), " + Sampling_frequency + "Hz, " + (Bitrate / 1000.0) + "kbps, " + Size + "BpF")
    'End Function

    ''/*** display last dts header ***/ 
    'Friend Function DTS_saveAnddisplayHeader() As String
    '    'saveHeader()
    '    Return DTS_displayHeader()
    'End Function


    Private ReadOnly WaveChunks() As Integer = _
    { _
     &H57415645, _
     &H63756520, _
     &H64617461, _
     &H66616374, _
     &H666D7420, _
     &H696E7374, _
     &H6C61626C, _
     &H6C697374, _
     &H6C747874, _
     &H6E6F7465, _
     &H706C7374, _
     &H736D706C _
    }

    Private INTEL As Boolean = False

    Private Function littleEndian(ByVal data() As Byte, ByVal offset As Integer, ByVal len As Integer, ByVal INTEL As Boolean) As Integer
        Dim value As Integer = 0
        For a As Integer = 0 To len - 1 '(int a=0; a<len; a++)
            value = value Or IIf(INTEL, ((&HFF And data(offset + a)) << (a * 8)), ((&HFF And data(offset + a)) << ((len - 1 - a) * 8)))
        Next

        Return value
    End Function

    Private Function littleEndian(ByVal data As Integer, ByVal len As Integer) As Integer
        If (Not INTEL) Then Return data
        If (len = 4) Then
            Return ((&HFF And Bitweise.RechtsOhne(data, 24)) Or (&HFF And Bitweise.RechtsOhne(data, 16)) << 8 Or (&HFF And Bitweise.RechtsOhne(data, 8)) << 16 Or (&HFF And data) << 24)
        Else
            Return ((&HFF And Bitweise.RechtsOhne(data, 8)) Or (&HFF And data) << 8)
        End If
    End Function


    '/*** parse RIFF_WAVE Header ***/ 
    Friend Function WAV_parseHeader(ByVal frame() As Byte, ByVal pos As Integer) As Integer
        INTEL = False
        If (frame(pos) <> &H52 OrElse frame(pos + 1) <> &H49 OrElse frame(pos + 2) <> &H46) Then Return -1

        If (frame(pos + 3) = &H46) Then
            INTEL = True
        ElseIf (frame(pos + 3) <> &H58) Then
            Return -2
        End If

        ID = IIf(INTEL, 0, 1)
        Emphasis = 0
        Protection_bit = 0 ^ 1

        Array.Sort(WaveChunks)

        If (littleEndian(frame, pos + 8, 4, False) <> WaveChunks(0)) Then Return -3

        Dim chunk As Integer = 0
        Dim chunksize As Integer = 0
        For a As Integer = pos + 12 To frame.Length - 4 - 1 '(int a=pos+12; a<frame.length-4; a++)
            chunk = littleEndian(frame, a, 4, False)
            If (Array.BinarySearch(WaveChunks, chunk) < 0) Then
                GoTo Continue1
            End If
            If (chunk = WaveChunks(4)) Then 'fmt chunk read info datas
                chunksize = littleEndian(frame, a + 4, 4, INTEL)
                Layer = littleEndian(frame, a + 8, 2, INTEL) ';   // Compression type (1=PCM)
                Channel = littleEndian(frame, a + 10, 2, INTEL) '; // channels
                Sampling_frequency = littleEndian(frame, a + 12, 4, INTEL) '; // samplerate
                Bitrate = littleEndian(frame, a + 16, 4, INTEL) * 8 '; // avg bits per second
                Mode = littleEndian(frame, a + 20, 2, INTEL) ' // block align, bytes per sample
                Size = littleEndian(frame, a + 22, 2, INTEL) '//bits per sample
                '//extrabits not of interest
            ElseIf (chunk = WaveChunks(2)) Then '//data chunk, sample data
                chunksize = littleEndian(frame, a + 4, 4, INTEL)
                Size_base = chunksize ' // length of whole sample data
                Emphasis = a + 8 '// real start of whole sample data
            Else
                chunksize = littleEndian(frame, a + 4, 4, INTEL)
            End If
            a += chunksize + 3
Continue1:
        Next

        'PTS low+high may exists in 'fact' of MPEG1audio !

        If (Bitrate < 1 OrElse Sampling_frequency < 1 OrElse Channel < 1) Then Return -4

        Padding_bit = 0
        Private_bit = 0
        Copyright = 0
        Original = 0
        Time_length = 90000.0 / Sampling_frequency

        Select Case (Layer)
            Case 1
                Mode_extension = 1
                Return 1
            Case &H50
                Mode_extension = 2
                Return 0
            Case &H55
                Mode_extension = 3
                Return 0
            Case &H2000
                Mode_extension = 4
                Return 0
            Case Else
                Mode_extension = 0
        End Select

        Return 0
    End Function

    Private ReadOnly wav_LSB_mode() As String = {"F", "X"}
    Private ReadOnly wav_compression() As String = {"", "PCM", "MPEG", "MPEG-L3", "AC3"}




    ''/*** display last wav header ***/ 
    'Friend Function WAV_displayHeader() As String
    '    Return ("RIF" + wav_LSB_mode(ID) + ", " + IIf(Mode_extension > 0, wav_compression(Mode_extension), "tag &H" & Layer.ToString("x2")) & ", " & Channel & "-ch, " & Sampling_frequency & "Hz, " & Size & "bit, " & (Bitrate / 1000.0) & "kbps")
    'End Function

    ''/*** display last wav header ***/ 
    'Friend Function WAV_saveAnddisplayHeader() As String
    '    'saveHeader()
    '    Return WAV_displayHeader()
    'End Function


    Friend Function getRiffHeader() As Byte()
        Dim RIFF() As Byte = { _
         &H52, &H49, &H46, &H46, 0, 0, 0, 0, _
         &H57, &H41, &H56, &H45, &H66, &H6D, &H74, &H20, _
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
         0, 0, 0, 0, &H64, &H61, &H74, &H61, _
         0, 0, 0, 0 _
        }
        Return RIFF
    End Function


    Friend Function GetAudioInfo(ByVal AudioFormat As AudioFormatType) As AudioInfo
        Dim AudioInfo As AudioInfo
        AudioInfo.Format = AudioFormat

        With AudioInfo
            Select Case .Format
                Case AudioFormatType.MPA
                    .FormatText = mpa_ID(ID) & " " & mpa_Layer(Layer)
                    .Channels = mpa_Mode(Mode)

                Case AudioFormatType.AC3
                    .FormatText = "AC3" & ac3_bsmod(Layer)
                    .Channels = ac3_mod(Mode) & ac3_lfe(1, 1 And Mode_extension) & "(" & ac3_channels(Mode) & ac3_lfe(0, 1 And Mode_extension) & ")" & ac3_dsurmod(Bitweise.RechtsOhne(Mode_extension, 1))

                Case AudioFormatType.DTS
                    .FormatText = "DTS"
                    .Channels = dts_acmod(Mode) + "(" + dts_channels(Mode) + ")"

                Case AudioFormatType.WAV
                    .FormatText = "RIF" + wav_LSB_mode(ID) + ", " + IIf(Mode_extension > 0, wav_compression(Mode_extension), "tag &H" & Layer.ToString("x2"))
                    .Channels = Channel & "-ch"
            End Select

            .Bitrate = Bitrate / 1000
            .Frequence = Sampling_frequency
        End With

        Return AudioInfo
    End Function
End Class