Imports System.IO

Public Enum FileType
    Unknown
    Audio
    Video
    VideoAudio
    PVA
    TransportStream
    Subpicture
End Enum

Public Class ScanFile
    ' 0 = Filetype not supported
    ' 1 = PVA (Video/Audio PES)
    ' 2 = MPEG-1 PS/SS (Video/Audio PES)
    ' 3 = MPEG-2 PS/SS (Video/Audio PES)
    ' 4 = PES (Video/Audio/TTX)
    ' 5 = PES (MPEG Audio)
    ' 6 = PES (private stream 1)
    ' 7 = ES (AC-3 Audio)
    ' 8 = ES (MPEG Audio)
    ' 9 = ES (MPEG Video)
    '10 = ES (AC-3 Audio) (psb. SMPTE)
    '11 = DVB/MPEG2 TS
    '12 = ES (DTS Audio)
    '13 = ES (DTS Audio) (psb. SMPTE)
    '14 = ES (RIFF Audio)
    '15 = ES (compressed RIFF Audio)
    '16 = ES (Subpicture 2-bit RLE)

    'Public addInfo As String = String.Empty

    Private pics As String = String.Empty

    'Auflistung der VideoStreams
    Private m_videos As ArrayList
    Public ReadOnly Property Videos(ByVal Index As Integer) As VideoInfo
        Get
            Return DirectCast(m_videos(Index), VideoInfo)
        End Get
    End Property

    Public ReadOnly Property Videos() As ArrayList
        Get
            Return m_videos
        End Get
    End Property

    Public ReadOnly Property VideoInfo() As VideoInfo
        Get
            If Not m_videos.Count = 0 Then
                Return CType(m_videos(0), VideoInfo)
            Else
                Return Nothing
            End If
        End Get
    End Property

    'Auflistung der AudioStreams
    Private m_audios As ArrayList
    Public ReadOnly Property Audios(ByVal Index As Integer) As AudioInfo
        Get
            Return DirectCast(m_audios(Index), AudioInfo)
        End Get
    End Property

    Public ReadOnly Property Audios() As ArrayList
        Get
            Return m_audios
        End Get
    End Property

    Public ReadOnly Property AudioInfo() As AudioInfo
        Get
            If Not m_audios.Count = 0 Then
                Return CType(m_audios(0), AudioInfo)
            Else
                Return Nothing
            End If
        End Get
    End Property

    'Infos zu einem TransportStream
    Private m_tsinfo As New TransportStreamInfo
    Public ReadOnly Property TSInfo() As TransportStreamInfo
        Get
            Return m_tsinfo
        End Get
    End Property

    Private m_ispes As Boolean = False
    Public ReadOnly Property IsPES() As Boolean
        Get
            Return Me.m_ispes
        End Get
    End Property

    Private audiobuffer As audiobuffer
    Private videobuffer As videobuffer

    Private hasVideo As Boolean = False
    Private nullpacket As Boolean = False
    Private vbasic(11) As Byte

    Private buffersize As Integer = 1024000

    Private m_filetype As FileType
    Public ReadOnly Property FileType() As FileType
        Get
            Return Me.m_filetype
        End Get
    End Property

    Public Sub New(ByVal FileName As String)
        Me.New(FileName, 0)
    End Sub

    Public Sub New(ByVal FileName As String, ByVal Pos As Int64)
        Initialize()
        Me.m_filetype = GetFileType(FileName)
    End Sub

    Private Sub Initialize()
        Me.m_videos = New ArrayList
        Me.m_audios = New ArrayList

        Me.audiobuffer = New AudioBuffer
        Me.videobuffer = New VideoBuffer
    End Sub

    Private Function GetFileType(ByVal FileName As String) As FileType
        Return GetFileType(FileName, 0)
    End Function

    Private Function GetFileType(ByVal FileName As String, ByVal Pos As Int64) As FileType
        Dim size As Long = 0

        Dim bs0 As Integer = buffersize / 100
        Dim bs1 As Integer = buffersize / 50
        Dim bs2 As Integer = buffersize / 10
        Dim bs3 As Integer = buffersize / 4
        Dim bs4 As Integer = buffersize - 65536

        Dim check(buffersize - 1) As Byte
        Dim bytecheck As MemoryStream = New MemoryStream

        Dim FileInfo As New FileInfo(FileName)
        size = FileInfo.Length
        Dim inStream As BinaryReader = New BinaryReader(FileInfo.Open(FileMode.Open))
        inStream.BaseStream.Position = Pos
        inStream.Read(check, 0, buffersize)
        inStream.Close()

        'riffcheck:
        If Me.RIFFAudio(check) Then Return FileType.Audio '14

        'supcheck:
        If Me.checkSup(check) Then Return FileType.Subpicture '16

        'mpegtscheck:
        If checkPMT(check) Then Return FileType.TransportStream '11

        'pvacheck:
        For a As Integer = 0 To bs1 - 1
            If (check(a) = &H41 AndAlso check(a + 1) = &H56 AndAlso check(a + 4) = &H55) Then
                Dim [next] As Integer = a + 8 + ((255 And check(a + 6)) << 8 Or (255 And check(a + 7)))

                If (check([next]) = &H41 AndAlso check([next] + 1) = &H56 AndAlso check([next] + 4) = &H55) Then
                    loadPVA(check, a)
                    Return FileType.PVA '1
                End If
            End If
        Next

        'mpgcheck:
        For a As Integer = 0 To bs2 - 1
            If (check(a) = 0 AndAlso check(a + 1) = 0 AndAlso check(a + 2) = 1 AndAlso check(a + 3) = &HBA) Then
                If ((&HC0 And check(a + 4)) = 0) Then
                    loadMPG2(check, a, False, VideoFormatType.MPEG1)
                    Return FileType.VideoAudio '2
                ElseIf ((&HC0 And check(a + 4)) = &H40) Then
                    loadMPG2(check, a, False, VideoFormatType.MPEG2)
                    Return FileType.VideoAudio '3
                End If
            End If
        Next

        'vdrcheck:
        For a As Integer = 0 To bs4 - 1
            If (check(a) = 0 AndAlso check(a + 1) = 0 AndAlso check(a + 2) = 1 AndAlso (&HF0 And check(a + 3)) = &HE0) Then
                Dim [next] As Integer = a + 6 + ((255 And check(a + 4)) << 8 Or (255 And check(a + 5)))

                If ([next] = a + 6 AndAlso (&HC0 And check(a + 6)) = &H80 AndAlso (&HC0 And check(a + 8)) = 0) Then
                    [next] = a
                    nullpacket = True
                End If

                If (check([next]) = 0 AndAlso check([next] + 1) = 0 AndAlso check([next] + 2) = 1) Then
                    loadMPG2(check, a, True, VideoFormatType.MPEG2)
                    Me.m_ispes = True
                    If Me.m_videos.Count = 1 AndAlso Me.m_audios.Count = 0 Then
                        Return FileType.Video
                    ElseIf Me.m_videos.Count = 0 AndAlso Me.m_audios.Count = 1 Then
                        Return FileType.Audio
                    Else
                        Return FileType.VideoAudio
                    End If
                End If
            End If
        Next

        'rawcheck:
        If Me.checkPES(check) Then
            Me.m_ispes = True
            Return FileType.Audio '5
        End If

        'audiocheck:
        If Me.DTSAudio(check) Then Return FileType.Audio '12
        If Me.AC3Audio(check) Then Return FileType.Audio '7
        If Me.MPEGAudio(check) Then Return FileType.Audio '8

        'mpvcheck:
        If Me.checkVid(check) Then Return FileType.Video '9

        check = Nothing
        Return FileType.Unknown
    End Function

    '/* DTS stuff taken from the VideoLAN project. */ 
    '/* Added by R One, 2003/12/18. */
    Private Function DTSAudio(ByVal check As Byte()) As Boolean
        For a As Integer = 0 To 10000
            If (audiobuffer.DTS_parseHeader(check, a) >= 0) Then
                For b As Integer = 0 To 14
                    If (audiobuffer.DTS_parseNextHeader(check, a + audiobuffer.Size + b) = 1) Then
                        If ((&HFF And check(a + audiobuffer.Size)) <= &H7F AndAlso (&HFF And check(a + audiobuffer.Size)) <> 0) Then
                            m_audios.Add(audiobuffer.GetAudioInfo(AudioFormatType.DTS))
                            Return True
                        End If
                    End If
                Next
            End If
        Next

        Return False
    End Function

    Private Function AC3Audio(ByVal check As Byte()) As Boolean
        For a As Integer = 0 To 10000
            If (audiobuffer.AC3_parseHeader(check, a) >= 0) Then
                For b As Integer = 0 To 16
                    If (audiobuffer.AC3_parseNextHeader(check, a + audiobuffer.Size + b) = 1) Then
                        If ((&HFF And check(a + audiobuffer.Size)) <= &H3F AndAlso (&HFF And check(a + audiobuffer.Size)) <> 0) Then
                            m_audios.Add(audiobuffer.GetAudioInfo(AudioFormatType.AC3))
                            Return True
                        End If
                    End If
                Next
            End If
        Next

        Return False
    End Function

    Private Function MPEGAudio(ByVal check As Byte()) As Boolean
        For a As Integer = 0 To 10000
            If (audiobuffer.MPA_parseHeader(check, a) >= 0) Then
                If (audiobuffer.MPA_parseNextHeader(check, a + audiobuffer.Size) >= 0) Then
                    m_audios.Add(audiobuffer.GetAudioInfo(AudioFormatType.MPA))
                    Return True
                End If
            End If
        Next

        Return False
    End Function

    Private Function RIFFAudio(ByVal check As Byte()) As Boolean
        For a As Integer = 0 To 10000
            Dim ERRORCODE As Integer = audiobuffer.WAV_parseHeader(check, a)

            If (ERRORCODE > -1) Then
                m_audios.Add(audiobuffer.GetAudioInfo(AudioFormatType.WAV))

                If (ERRORCODE > 0) Then
                    Return True

                ElseIf (audiobuffer.Mode_extension > 1) Then
                    Exit For

                Else
                    Return True
                End If
            End If
        Next

        Return False
    End Function


    Private Function loadPES(ByVal check As Byte(), ByVal a As Integer) As Byte()
        Dim bytecheck As MemoryStream = New MemoryStream
        Dim jump, offs, len As Integer

        Dim mpg1 As Boolean

        While a < check.Length - 8
            jump = (&HFF And check(a + 4)) << 8 Or (&HFF And check(a + 5))
            mpg1 = (&H80 And check(a + 6)) = 0
            offs = a + 6 + IIf(Not mpg1, 3 + (&HFF And check(a + 8)), 0)
            len = jump - IIf(Not mpg1, 3 + (&HFF And check(a + 8)), 0)

            If (offs + len > check.Length) Then Exit While

            bytecheck.Write(check, offs, len)

            a += 6 + jump
        End While

        Return bytecheck.ToArray()
    End Function


    Private Sub loadMPG2(ByVal check As Byte(), ByVal b As Integer, ByVal vdr As Boolean, ByVal VideoFormat As VideoFormatType)
        Dim table As Collections.Specialized.ListDictionary = New Collections.Specialized.ListDictionary
        Dim scanobject As ScanObject

        Dim str As String

        Dim jump As Integer = 1
        Dim id As Integer

        'mpg2check:
        Dim a As Integer = b
        While a < 500000
            If (check(a) = 0 AndAlso check(a + 1) = 0 AndAlso check(a + 2) = 1) Then
                id = &HFF And check(a + 3)
                str = id 'String.ValueOf(id)

                If (id = &HBA) Then
                    jump = IIf((&HC0 And check(a + 4)) = 0, 12, 14)

                    'video mpg  e0..0f
                ElseIf ((&HF0 And id) = &HE0) Then
                    jump = IIf(nullpacket, 2048, (6 + ((&HFF And check(a + 4)) << 8 Or (&HFF And check(a + 5)))))

                    If (Not table.Contains(str)) Then table.Add(str, New ScanObject(id))

                    scanobject = DirectCast(table.Item(str), ScanObject)

                    scanobject.write(check, a + 6 + IIf(Not VideoFormatType.MPEG1, 3 + (&HFF And check(a + 8)), 0), jump - IIf((Not VideoFormatType.MPEG1), 6 - 3 - (&HFF And check(a + 8)), 0))

                    'audio mpg  c0..df
                ElseIf ((&HE0 And id) = &HC0) Then
                    jump = 6 + ((&HFF And check(a + 4)) << 8 Or (&HFF And check(a + 5)))

                    If (Not table.Contains(str)) Then table.Add(str, New ScanObject(id))

                    scanobject = DirectCast(table.Item(str), ScanObject)

                    scanobject.write(check, a, jump)

                    'private bd
                ElseIf (id = &HBD) Then
                    jump = 6 + ((&HFF And check(a + 4)) << 8 Or (&HFF And check(a + 5)))

                    If (check(a + 8) <> &H24 AndAlso Bitweise.RechtsOhne((&HF0 And check(a + 9 + &H24)), 4) <> 1) Then
                        '//DM10032004 081.6 int18 add
                        If (Not vdr) Then
                            id = &HFF And check(a + 9 + (&HFF And check(a + 8)))
                            str = id 'String.valueOf(id)

                            check(a + 8) = 4 + (&HFF And check(a + 8))
                        End If

                        If (Not table.Contains(str)) Then table.Add(str, New ScanObject(id))

                        scanobject = DirectCast(table.Item(str), ScanObject)

                        scanobject.write(check, a, jump)
                    End If
                Else
                    Select Case id
                        Case &HBB, &HBC, &HBE, &HBF, &HF0 To &HFF
                            jump = 6 + ((&HFF And check(a + 4)) << 8 Or (&HFF And check(a + 5)))
                        Case Else
                            jump = 1
                    End Select
                End If
            Else
                jump = 1
            End If

            a += jump
        End While


        For Each key As String In table.Keys
            id = Int(key)

            scanobject = DirectCast(table.Item(key), ScanObject)

            If (&HF0 And id) = &HE0 Then
                Try
                    checkVid(scanobject.getData(), VideoFormat)
                Catch e As Exception
                End Try
            Else
                Try
                    checkPES(scanobject.getData())
                Catch e As Exception
                End Try
            End If
        Next

        table.Clear()

        Return
    End Sub


    Private Sub loadPVA(ByVal check As Byte(), ByVal a As Integer)
        Dim table As Collections.Specialized.ListDictionary = New Collections.Specialized.ListDictionary
        Dim scanobject As ScanObject

        Dim str As String
        Dim jump As Integer
        Dim id As Integer

        While (a < 550000)
            jump = (&HFF And check(a + 6)) << 8 Or (&HFF And check(a + 7))

            If (a + 8 + (Bitweise.RechtsOhne(1 And check(a + 5), 4) * 4) + jump > 700000) Then Exit While

            id = &HFF And check(a + 2)
            str = id 'String.valueOf(id)

            If (Not table.Contains(str)) Then table.Add(str, New ScanObject(id))

            scanobject = DirectCast(table.Item(str), ScanObject)

            Select Case id
                Case 1
                    scanobject.write(check, a + 8 + (Bitweise.RechtsOhne(1 And check(a + 5), 4) * 4), jump)

                Case Else
                    scanobject.write(check, a + 8, jump)
            End Select

            a += 8 + jump
        End While

        For Each key As String In table.Keys '(Enumeration n = table.keys(); n.hasMoreElements() ; )
            scanobject = DirectCast(table.Item(key), ScanObject)

            If (str.Equals("1")) Then
                Try
                    checkVid(scanobject.getData())
                Catch e As Exception
                End Try
            Else
                Try
                    checkPES(scanobject.getData())
                Catch e As Exception
                End Try
            End If
        Next

        table.Clear()

        Return
    End Sub


    Private Function checkPES(ByVal check As Byte()) As Boolean
        'rawcheck:
        For a As Integer = 0 To 8000
            If (check(a) = 0 AndAlso check(a + 1) = 0 AndAlso check(a + 2) = 1) Then
                If ((&HE0 And check(a + 3)) = &HC0 OrElse check(a + 3) = &HBD) Then
                    Dim [next] As Integer = a + 6 + ((&HFF And check(a + 4)) << 8 Or (&HFF And check(a + 5)))
                    If (check([next]) = 0 AndAlso check([next] + 1) = 0 AndAlso check([next] + 2) = 1) Then
                        If ((&HE0 And check(a + 3)) = &HC0 AndAlso (&HE0 And check(a + 3)) = (&HE0 And check([next] + 3))) Then
                            MPEGAudio(loadPES(check, a))
                            Return True
                        ElseIf (check(a + 3) = &HBD AndAlso check(a + 3) = check([next] + 3)) Then
                            Dim buffer As Byte() = loadPES(check, a) '; //DM19122003 081.6 int07 changed
                            If Not AC3Audio(buffer) Then DTSAudio(buffer)
                            Return True
                        End If
                    End If
                End If
            End If
        Next
        Return False
    End Function

    Private Function checkSup(ByVal check As Byte()) As Boolean
        'supcheck:
        For a As Integer = 0 To 10000
            If (check(a) = &H53 AndAlso check(a + 1) = &H50) Then
                Dim supframe_size As Integer = (&HFF And check(a + 10)) << 8 Or (&HFF And check(a + 11))
                Dim supframe_link As Integer = (&HFF And check(a + 12)) << 8 Or (&HFF And check(a + 13))
                Dim supframe_check As Integer = (&HFF And check(a + 12 + supframe_link)) << 8 Or (&HFF And check(a + 13 + supframe_link))
                Dim supframe_check2 As Integer = (&HFF And check(a + 36 + supframe_link)) << 8 Or (&HFF And check(a + 37 + supframe_link))

                If (supframe_link = supframe_check - 24 AndAlso supframe_check = supframe_check2) Then
                    Dim b As Integer = a + 14 + supframe_link
                    Dim c As Integer = b + 24
                    Dim d As Integer
                    Dim xa As Integer
                    Dim xe As Integer
                    Dim ya As Integer
                    Dim ye As Integer
                    For d = b To c - 1 '(d=b; d<c; d++)
                        Select Case (&HFF And check(d))
                            Case 1
                                d += 1
                            Case 2
                                d += 24
                            Case 3, 4
                                d += 2
                            Case 6
                                d += 4
                            Case 5
                                xa = (&HFF And check(++d)) << 4 Or Bitweise.RechtsOhne(&HF0 And check(++d), 4)
                                xe = (&HF And check(d)) << 8 Or (&HFF And check(++d))
                                ya = (&HFF And check(++d)) << 4 Or Bitweise.RechtsOhne(&HF0 And check(++d), 4)
                                ye = (&HF And check(d)) << 8 Or (&HFF And check(++d))
                                pics = "up.left x" & xa & ",y" & ya & " @ size " & (xe - xa + 1) & "*" & (ye - ya + 1) '//DM05052004 081.7 int02 changed,fix
                                Exit For
                            Case Else
                                Exit For
                        End Select
                    Next

                    Return True
                End If
            End If
        Next

        Return False
    End Function

    Private Function checkVid(ByVal check As Byte()) As Boolean
        Return checkVid(check, VideoFormatType.MPEG2)
    End Function

    Private Function checkVid(ByVal check As Byte(), ByVal VideoFormat As VideoFormatType) As Boolean
        Dim bytecheck As MemoryStream = New MemoryStream

        'mpvcheck:
        For a As Integer = 0 To (check.Length - 629) '(int a=0; a<check.length-630; a++) {
            If (check(a) = 0 AndAlso check(a + 1) = 0 AndAlso check(a + 2) = 1 AndAlso check(a + 3) = &HB3) Then
                For b As Integer = 7 To 599 '(int b=7;b<600;b++) 
                    If (check(a + b) = 0 AndAlso check(a + b + 1) = 0 AndAlso check(a + b + 2) = 1 AndAlso check(a + b + 3) = &HB8) Then
                        hasVideo = True
                        Array.Copy(check, a, vbasic, 0, 12)
                        bytecheck.Write(check, a, 20)

                        m_videos.Add(videobuffer.GetVideoInfo(bytecheck.ToArray(), VideoFormat))
                        Return True
                    End If
                Next
            End If
        Next

        Return False
    End Function

    Private Function checkPMT(ByVal check As Byte()) As Boolean
        Dim bytecheck As MemoryStream = New MemoryStream

        'tscheck:
        For a As Integer = 0 To check.Length - 1000 '( ; a<check.length-1000; a++) {
            If (check(a) = &H47 AndAlso check(a + 188) = &H47 AndAlso check(a + 376) = &H47 AndAlso check(a + 564) = &H47 AndAlso check(a + 752) = &H47) Then
                If (check(a) = &H47 AndAlso check(a + 188) = &H47 AndAlso check(a + 376) = &H47) Then
                    If ((&H30 And check(a + 3)) = &H10 AndAlso check(a + 4) = 0 AndAlso check(a + 5) = 2 AndAlso (&HF0 And check(a + 6)) = &HB0) Then
                        bytecheck.Write(check, a + 4, 184)
                        Dim pmtpid As Integer = (&H1F And check(a + 1)) << 8 Or (&HFF And check(a + 2))
                        If (bytecheck.Length() < 188) Then
                            a += 188

                            While a < check.Length - 500 '( ; a<check.length-500; a++) {
                                If (check(a) = &H47 AndAlso check(a + 188) = &H47 AndAlso check(a + 376) = &H47) Then
                                    If (((&H1F And check(a + 1)) << 8 Or (&HFF And check(a + 2))) = pmtpid AndAlso (&H40 And check(a + 1)) = 0 AndAlso (&H30 And check(a + 3)) = &H10) Then
                                        bytecheck.Write(check, a + 4, 184)

                                        If (bytecheck.Length() > 188) Then
                                            Exit While
                                        End If
                                    Else
                                        a += 187
                                    End If
                                End If
                                a += 1
                            End While
                        End If

                        Dim pmt As Byte() = bytecheck.ToArray()
                        If (pmt.Length > 5) Then
                            Dim sid As Integer = (&HFF And pmt(4)) << 8 Or (&HFF And pmt(5))
                            Me.m_tsinfo.SID = sid
                            Me.m_tsinfo.PMT = pmtpid
                        End If

                        Dim pmt_len As Integer = (&HF And pmt(2)) << 8 Or (&HFF And pmt(3)) '  //DM30122003 081.6 int10 add

                        '//DM30122003 081.6 int10 changed
                        '//DM10032004 081.6 int18 changed  , &hF  and  pmt(a+3)<<4 or &hFF  and  pmt(a+4)

                        Dim b As Integer = 8
                        Dim r As Integer = 8
                        While b < pmt_len - 4 AndAlso b < pmt.Length - 6 '(int b=8, r=8; b < pmt_len-4  andalso b < pmt.length-6; b++)

                            r = b
                            If ((&HE0 And pmt(b + 1)) = &HE0) Then
                                Dim pid As Integer = (&H1F And pmt(b + 1)) << 8 Or (&HFF And pmt(b + 2))

                                Select Case (&HFF And pmt(b))
                                    Case 1, 2
                                        Dim tmp As Integer = b + 4 + (&HFF And pmt(b + 4))
                                        getIsoLanguage(pmt, b + 5, tmp, pid, 2)
                                        b = tmp

                                    Case 3, 4
                                        Dim tmp As Integer = b + 4 + (&HFF And pmt(b + 4))
                                        getIsoLanguage(pmt, b + 5, tmp, pid, 4)
                                        b = tmp

                                    Case &H80 To &H83, 6 '&h81:private data of AC3 in ATSC
                                        Dim tmp As Integer = b + 4 + (&HFF And pmt(b + 4))
                                        getIsoLanguage(pmt, b + 5, tmp, pid, 6)
                                        b = tmp

                                    Case Else
                                        b += 4 + (&HFF And pmt(b + 4))
                                End Select

                                If (b < 0) Then
                                    b = r
                                End If
                            End If
                            b += 1
                        End While
                        Return True
                    Else
                        a += 187
                    End If
                End If
            End If
        Next

        Return False
    End Function


    '//DM10032004 081.6 int18 new
    '//DM04052004 081.7 int02 fix
    Private Sub getIsoLanguage(ByVal check() As Byte, ByVal off As Integer, ByVal [end] As Integer, ByVal pid As Integer, ByVal type As Integer)
        While off < [end] AndAlso off < check.Length
            Select Case (&HFF And check(off))
                Case &H59  'DVB subtitle 
                    type = &H59
                    Exit While

                Case &H56  'Teletext
                    type = &H56
                    Exit While

                Case &H5   'Registration descriptor
                Case Else
                    off += 1
                    off += (&HFF And check(off))
            End Select
            off += 1
        End While

        Select Case type
            Case &H59 '  //DM28042004 081.7 int02 add
                m_tsinfo.AddPID(PIDType.Subpicture, pid)

            Case &H56
                m_tsinfo.AddPID(PIDType.Teletext, pid)

            Case 2
                m_tsinfo.AddPID(PIDType.Video, pid)

            Case 4
                m_tsinfo.AddPID(PIDType.Audio, pid)

            Case Else
                m_tsinfo.AddPID(PIDType.Audio, pid)
        End Select
    End Sub



    Private Class ScanObject
        Private buf As MemoryStream
        Private id As Integer
        Private type As Integer

        Public Sub New()
            buf = New MemoryStream
            id = 0
        End Sub

        Public Sub New(ByVal id As Integer)
            buf = New MemoryStream
            Me.id = id
        End Sub

        Public Shadows Function [getType]() As Integer
            Return type
        End Function

        Public Sub write(ByVal data() As Byte)
            buf.Write(data, 0, data.Length)
        End Sub

        Public Sub write(ByVal data() As Byte, ByVal offset As Integer, ByVal length As Integer)
            buf.Write(data, offset, length)
        End Sub

        Public Function getData() As Byte()
            buf.Flush()
            Return buf.ToArray()
        End Function

        Public Sub reset()
            buf = Nothing
        End Sub
    End Class
End Class